/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module PicZoomInAnDout(
    clk               ,
    rst_n             ,
    mode              ,
    key_en            ,
    cpu_ZoomIn        ,   
    cpu_ZoomOut       ,
                
    req_h             ,//h_cnt - THB   
    req_v             ,//v_cnt - TVB
    pic_data          
    );

    //LCD显示屏居中
    parameter       HDE_CENTRE  = 400      ;//800/2
    parameter       VDE_CENTRE  = 240      ;//480/2


    parameter      DATA_W    =         8;
    parameter      KEY_W     =         4;

    //输入信号定义
    input                       clk          ;
    input                       rst_n        ;
    input                       mode         ;
    input                       cpu_ZoomIn   ;
    input                       cpu_ZoomOut  ;
    input   [KEY_W-1 :0]        key_en       ;

    //输出信号定义
    input   [10:0]              req_h        ;
    input   [ 9:0]              req_v        ;
    output  [23:0]              pic_data     ;

    //输出信号reg定义
    reg     [23:0]              pic_data     ;
   
    //中间信号
    reg     [15:0]              logo_rom_addr   ;
    reg                         logo_rom_area   ;
    wire    [7:0]               logo_rom_data   ;
    reg     [17:0]              e_rom_addr      ;
    reg                         e_rom_area      ;
    reg      [2:0]              e_rom_addr_low  ;
    reg                         e_dataout       ;
    wire    [7:0]               e_rom_data      ;
                                                 
    reg     [ 2:0]              size            ; 
    reg     [ 2:0]              size_ff0        ;

    wire    [9:0]               len_size        ;
    wire    [9:0]               wid_size        ;
    wire    [9:0]               e_x0            ;
    wire    [9:0]               e_x1            ;
    wire    [9:0]               e_y0            ;
    wire    [9:0]               e_y1            ;
    wire    [9:0]               x               ;
    wire    [9:0]               y               ;

/******************************************************/




always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        size <= 0;
    end
    else if(mode==1)begin
        if(cpu_ZoomOut)begin
            if(size!=3)
                size <= size + 1;
        end
        else if(cpu_ZoomIn)begin
            if(size!=0)
                size <= size - 1;
        end
    end
    else if(mode==0)begin
        if(key_en==4'b0010)begin
            if(size!=3)
                size <= size + 1;
        end
        else if(key_en==4'b0001)begin
            if(size!=0)
                size <= size - 1;
        end
    end
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        size_ff0 <= 0;
    end
    else if(req_h==(HDE_CENTRE-200) && req_v==(VDE_CENTRE-150)) begin
        size_ff0 <= size;
    end
end

assign len_size = 400 >> size_ff0;//缩小多少倍
assign wid_size = 300 >> size_ff0;


assign   e_x0 = (HDE_CENTRE-len_size[9:1]);
assign   e_x1 = (HDE_CENTRE+len_size[9:1]);
assign   e_y0 = (VDE_CENTRE-wid_size[9:1]);
assign   e_y1 = (VDE_CENTRE+wid_size[9:1]);

assign x = (req_h-e_x0)<<size;
assign y = (req_v-e_y0)<<size;




always  @(*)begin
    e_rom_area = req_h >=(e_x0+5) && req_h < e_x1 && req_v >= e_y0 && req_v < (e_y1+5);
end

always  @(*)begin
    e_rom_addr = x + 400*y;
end



/******************************************************/
   
//120*55
always  @(*)begin
    logo_rom_area = req_h >=0 && req_h < 119 && req_v >= 0 && req_v < 54;
end

always  @(*)begin
    logo_rom_addr = req_h + 120*req_v;
end


/******************************************************/

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        e_rom_addr_low <= 0;
    end
    else begin
        e_rom_addr_low <= e_rom_addr[2:0];
    end
end

always  @(*)begin
    if(e_rom_area)
        e_dataout = ~e_rom_data[7-e_rom_addr_low];
    else
        e_dataout = 1;
end

always  @(posedge clk or negedge rst_n)begin
    if(rst_n==1'b0)begin
        pic_data <= 0;
    end
    else if(e_rom_area)begin
        pic_data <= {24{e_dataout}};
    end
    else if(logo_rom_area)begin
        pic_data <= {logo_rom_data[7:5],5'b11111,logo_rom_data[4:2],5'b11111,logo_rom_data[1:0],6'b111111} ;
    end
end

/******************************************************/
mdyRom#(.MIF("../src/data/logo.mif"),.DEP(8192),.D_W(8))  u_fpga_rom(
    .address  (logo_rom_addr   ),
    .clock    (clk   ),
    .q        (logo_rom_data   )
);

mdyRom#(.MIF("../src/data/e.mif"),.DEP(16384),.D_W(8))  u_e_rom(
    .address  (e_rom_addr[16:3]   ),
    .clock    (clk   ),
    .q        (e_rom_data   )
);

    endmodule


