/************************************************************************************
The code is designed and produced by MDY Science and Education Co., Ltd, which has the entire ownership. It is only for personal learning, which cannot be used for commercial or profit-making purposes without permission.

    MDY's Mission: Develop Chip Talents and Realize National Chip Dream.

    We sincerely hope that our students can learn the real IC / FPGA code through our standard and rigorous code.

    For more FPGA learning materials, please visit the Forum: http://fpgabbs.com/ and official website: http://www.mdy-edu.com/index.html 

    *************************************************************************************/

module top_mdyLcdPicZoomInAndOut(
    clk       ,
    rst_n     ,
    key       ,
    uart_rxd  ,
          
    uart_txd  ,
    lcd_hs    ,
    lcd_vs    ,
    lcd_de    ,
    lcd_dat   ,
    lcd_clk    
    );

    parameter   PICTURE_W   = 24  ;
    parameter   KEY_W       =  4  ;


    input                   clk          ;//50MHz
    input                   rst_n        ;
    input   [KEY_W-1 :0]    key          ;
    input                   uart_rxd     ;
                                  
    output                  uart_txd     ;
    output                  lcd_hs       ;
    output                  lcd_vs       ;
    output                  lcd_de       ;
    output  [PICTURE_W-1:0] lcd_dat      ;
    output                  lcd_clk      ;

    wire                     uart_txd    ;
    wire                     lcd_hs      ;
    wire                     lcd_vs      ;
    wire                     lcd_de      ;
    wire   [PICTURE_W-1:0]   lcd_dat     ;
    wire                     lcd_clk     ;

    wire                     clk_0       ;
    wire    [23:0]           pic_data    ;
    wire    [10:0]           req_x       ;
    wire    [ 9:0]           req_y       ;
    wire                     frame_start ;
    wire    [KEY_W-1:0]      key_vld     ;
    wire                     frame_pos   ;
    wire    [63:0]           uart_dout         ;
    wire                     uart_dout_vld     ;
    wire    [63:0]           cfgCtrl_dout      ;
    wire                     cfgCtrl_dout_vld  ;

    `include "mdyCfg_wire.v"

/******************   PLL模块  *******************************/

mdyPllAltera#(.C0_M(4),.C0_D(5)) u1_pll_40m(
    .areset    (~rst_n   ), //高电平有效
    .inclk0    (clk     ),
    .c0        (clk_0   ),
    .c1        (     ),
    .c2        (     ),
    .c3        (     ),
    .c4        (     ),
    .locked    (     ) 

);

/******************   LCD驱动模块  *******************************/

mdyLcd#(.D_DLY(1))  u2_lcd_driver(
   .clk          (clk_0       ),//40MHz
   .rst_n        (rst_n       ),
   .ack_data     (pic_data    ), 
                          
   .req_x        (req_x       ),  
   .req_y        (req_y       ),  
   .req_en       (    ),                   
   .req_addr     (    ), 
   
   .hys          (lcd_hs         ),  
   .vys          (lcd_vs         ),  
   .lcd_de       (lcd_de      ),                   
   .lcd_rgb      (lcd_dat     ), 
   .lcd_dclk     (lcd_clk    ), 
   
   .frame_start  (frame_start ) 
    );

/******************  功能模块  *******************************/

PicZoomInAnDout u3_PicZoomInAnDout(
   .clk              (clk_0   ),
   .rst_n            (rst_n   ),
   .mode             (MODE_SELECT_en),
   .key_en           (key_vld ),
   .cpu_ZoomIn       (SOFTWARE_CTRL_in),
   .cpu_ZoomOut      (SOFTWARE_CTRL_out),

   .req_h            (req_x   ),   
   .req_v            (req_y   ),
   .pic_data         (pic_data       ) 
    
    );


/*********************  按键模块 ****************************/

 mdyKey#(.DATA_W(24),.TIME_20MS(8_00_000)) u4_mdykey(
    .clk        (clk_0   ),
    .rst_n      (rst_n   ),
    .key_in     (key  ),
    .key_vld    (key_vld )
);
/*********************  边沿检测模块  ****************************/

mdyGetEdge  u5_GetEdge(
    .clk              (clk_0   ),
    .rst_n            (rst_n   ),
    .cfg_init_value   (0   ),
    .din              (frame_start   ),
    .dout_pos         (frame_pos),
    .dout_neg         (   ), 
    .dout_pos_reg     (   ), 
    .dout_neg_reg     (   ) 
    );

/*********************  1s统计模块  ****************************/

mdyStatic_1S    u6_Static_1S(
    .clk          (clk_0   ),
    .rst_n        (rst_n   ),
    .cfg_num_1s   (50_000_000   ),
    .cfg_add_1s   (1            ),
    .din_vld      (frame_pos    ),
    .sta_1s       (DATA_FRAME_data   ),//32bit
    .sta_rt       (   )  
);

/*********************  指令模块  ****************************/

top_uart_cfg#(.BPS(4167)) u7_top_uart_cfg(
    .clk      (clk_0             ),
    .rst_n    (rst_n             ),
    .cfg_head (16'h55d5          ),
    .rx       (uart_rxd           ),
    .tx       (uart_txd           ),
    .din      (cfgCtrl_dout      ),//s2p
    .din_vld  (cfgCtrl_dout_vld  ),
    .din_rdy  (                  ),
    .dout     (uart_dout         ),//p2s
    .dout_vld (uart_dout_vld     )    

);

mdyCfgCtrl u8_mdyCfgCtrl(
    `include "mdyCfg_inst.v"
    .clk         (clk_0              ),
    .rst_n       (rst_n              ),
    .din         (uart_dout          ),
    .din_vld     (uart_dout_vld      ),
    .dout        (cfgCtrl_dout       ),
    .dout_vld    (cfgCtrl_dout_vld   )  

);

/*************************************************/
endmodule

